=begin
**
** Copyright (C) 2004-2005 Trolltech AS. All rights reserved.
**
** This file is part of the example classes of the Qt Toolkit.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  Please review the following information to ensure GNU
** General Public Licensing requirements will be met:
** http://www.trolltech.com/products/qt/opensource.html
**
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://www.trolltech.com/products/qt/licensing.html or contact the
** sales department at sales@trolltech.com.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**

** Translated to QtRuby by Richard Dale
=end

require 'Qt'
require 'treemodel.rb'

app = Qt::Application.new(ARGV)

file = Qt::File.new("default.txt")
file.open(Qt::IODevice::ReadOnly)
model = TreeModel.new(file.readAll)
file.close

unsortedView = Qt::TreeView.new
unsortedView.model = model
unsortedView.windowTitle = Qt::Object.tr("Unsorted Data")
unsortedView.show

sortingModel = Qt::SortFilterProxyModel.new
sortingModel.sourceModel = model

sortedView = Qt::TreeView.new
sortedView.model = sortingModel
sortedView.windowTitle = Qt::Object.tr("Sorted Data")
sortedView.header.setSortIndicator(1, Qt::AscendingOrder)
sortedView.header.sortIndicatorShown = true
sortedView.header.clickable = true
sortedView.show

# Qt::Internal::setDebug Qt::QtDebugChannel::QTDB_VIRTUAL
app.exec
